from unittest import TestCase

from mock import patch

from hestia.logging_utils import log_spec


class TestLoggingUtils(TestCase):
    def test_has_timestamp(self):
        self.assertEqual(
            log_spec(log_line="foo", timestamp="2018-12-11 10:24:57 UTC"),
            "2018-12-11 10:24:57 UTC -- foo",
        )

    def test_has_no_timestamp(self):
        with patch("hestia.logging_utils.now") as now_patch:
            log_spec(log_line="foo")

        self.assertEqual(now_patch.call_count, 1)

    def test_log_line_has_datetime(self):
        self.assertEqual(
            log_spec(log_line="2018-12-11 10:24:57 UTC foo", check_timestamp=True),
            "2018-12-11 10:24:57 UTC -- foo",
        )

    def test_log_line_has_iso_datetime(self):
        self.assertEqual(
            log_spec(
                log_line="2018-12-11T08:49:07.163495183Z foo", check_timestamp=True
            ),
            "2018-12-11 08:49:07 UTC -- foo",
        )

    def test_log_line_has_level(self):
        self.assertEqual(
            log_spec(
                log_line="2018-12-11T08:49:07.163495183Z foo",
                check_timestamp=True,
                log_level="INFO",
            ),
            "2018-12-11 08:49:07 UTC INFO -- foo",
        )



import logging
import os
import shutil

_logger = logging.getLogger("hestia.paths")

# noqa


def check_or_create_path(path: str = None) -> None:
    if not os.path.exists(path):
        os.makedirs(path)


def delete_path(path: str) -> None:
    if not os.path.exists(path):
        return
    try:
        if os.path.isfile(path):
            os.remove(path)
        else:
            shutil.rmtree(path)
    except OSError:
        _logger.warning("Could not delete path `%s`", path)


def create_path(path: str) -> None:
    try:
        os.mkdir(path)
    except FileExistsError:
        pass
    except OSError as e:
        _logger.warning("Could not create path `%s`, exception %s", path, e)


def get_tmp_path(path: str) -> str:
    return os.path.join("/tmp", path)


def create_tmp_dir(dir_name: str) -> None:
    create_path(get_tmp_path(dir_name))


def delete_tmp_dir(dir_name: str) -> None:
    delete_path(get_tmp_path(dir_name))


def copy_to_tmp_dir(path: str, dir_name: str) -> str:
    tmp_path = get_tmp_path(dir_name)
    if os.path.exists(tmp_path):
        return tmp_path
    try:
        shutil.copytree(path, tmp_path)
    except FileExistsError as e:
        _logger.warning("Path already exists `%s`, exception %s", path, e)
    return tmp_path



def memoize(func):
    """
    Provides memoization for methods on a specific instance.
    Results are cached for given parameter list.

    See also: http://en.wikipedia.org/wiki/Memoization

    N.B. The cache object gets added to the instance instead of the global scope.
    Therefore cached results are restricted to that instance.
    The cache dictionary gets a name containing the name of the decorated function to
    avoid clashes.

    Example:

        class MyClass(object):
            @memoize
            def foo(self, a, b):
                return self._do_calculation(a, b)

    HINT: - The decorator does not work with keyword arguments.
    """

    cache_name = "__CACHED_{}".format(func.__name__)

    def wrapper(self, *args):
        cache = getattr(self, cache_name, None)
        if cache is None:
            cache = {}
            setattr(self, cache_name, cache)
        if args not in cache:
            cache[args] = func(self, *args)
        return cache[args]

    return wrapper



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import pytz

from datetime import date, datetime

from hestia.exceptions import HestiaException

try:
    from django.conf import settings
except ImportError:
    raise HestiaException("This module depends on django.")


class DateTimeFormatterException(Exception):
    pass


class DateTimeFormatter(object):
    """
    The `DateTimeFormatter` class implements a utility used to create
    timestamps from strings and vice-versa.
    """

    DATE_FORMAT = "%Y-%m-%d"
    DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
    DATETIME_HOUR_FORMAT = "%Y-%m-%d %H:%M"

    @classmethod
    def format_date(cls, timestamp):
        """
        Creates a string representing the date information provided by the
        given `timestamp` object.
        """
        if not timestamp:
            raise DateTimeFormatterException(
                "timestamp must a valid string {}".format(timestamp)
            )

        return timestamp.strftime(cls.DATE_FORMAT)

    @classmethod
    def format_datetime(cls, timestamp):
        """
        Creates a string representing the date and time information provided by
        the given `timestamp` object.
        """
        if not timestamp:
            raise DateTimeFormatterException(
                "timestamp must a valid string {}".format(timestamp)
            )

        return timestamp.strftime(cls.DATETIME_FORMAT)

    @classmethod
    def extract_date(cls, date_str):
        """
        Tries to extract a `datetime` object from the given string, expecting
        date information only.

        Raises `DateTimeFormatterException` if the extraction fails.
        """
        if not date_str:
            raise DateTimeFormatterException(
                "date_str must a valid string {}.".format(date_str)
            )

        try:
            return cls._extract_timestamp(date_str, cls.DATE_FORMAT)
        except (TypeError, ValueError):
            raise DateTimeFormatterException("Invalid date string {}.".format(date_str))

    @classmethod
    def extract_datetime(cls, datetime_str):
        """
        Tries to extract a `datetime` object from the given string, including
        time information.

        Raises `DateTimeFormatterException` if the extraction fails.
        """
        if not datetime_str:
            raise DateTimeFormatterException("datetime_str must a valid string")

        try:
            return cls._extract_timestamp(datetime_str, cls.DATETIME_FORMAT)
        except (TypeError, ValueError):
            raise DateTimeFormatterException(
                "Invalid datetime string {}.".format(datetime_str)
            )

    @classmethod
    def extract_datetime_hour(cls, datetime_str):
        """
        Tries to extract a `datetime` object from the given string, including only hours.

        Raises `DateTimeFormatterException` if the extraction fails.
        """
        if not datetime_str:
            raise DateTimeFormatterException("datetime_str must a valid string")

        try:
            return cls._extract_timestamp(datetime_str, cls.DATETIME_HOUR_FORMAT)
        except (TypeError, ValueError):
            raise DateTimeFormatterException(
                "Invalid datetime string {}.".format(datetime_str)
            )

    @classmethod
    def extract(cls, timestamp_str):
        """
        Tries to extract a `datetime` object from the given string. First the
        datetime format is tried, if it fails, the date format is used for
        extraction.

        Raises `DateTimeFormatterException` if the extraction fails.
        """
        if not timestamp_str:
            raise DateTimeFormatterException(
                "timestamp_str must a valid string {}".format(timestamp_str)
            )

        if isinstance(timestamp_str, (date, datetime)):
            return timestamp_str

        try:
            return cls.extract_datetime(timestamp_str)
        except DateTimeFormatterException:
            pass

        try:
            return cls.extract_datetime_hour(timestamp_str)
        except DateTimeFormatterException:
            pass

        try:
            return cls.extract_date(timestamp_str)
        except DateTimeFormatterException as e:
            raise DateTimeFormatterException(e)

    @staticmethod
    def _extract_timestamp(timestamp_str, dt_format):
        timestamp = datetime.strptime(timestamp_str, dt_format)
        timestamp = timestamp.replace(tzinfo=pytz.timezone(settings.TIME_ZONE))
        return timestamp



from unittest import TestCase

from hestia.units import to_percentage


class ToPercentageTest(TestCase):
    """A test case for the `to_percentage`."""

    def test_works_as_expected_for_valid_values(self):
        test_data = [
            (0, "0%"),
            (0.25, "25%"),
            (-0.25, "-25%"),
            (12, "1200%"),
            (0.123, "12.3%"),
            (0.12345, "12.35%"),
            (0.12001, "12%"),
            (0.12101, "12.1%"),
            ("0", "0%"),
            ("0.25", "25%"),
        ]
        for value, expected in test_data:
            result = to_percentage(value)
            self.assertEqual(result, expected)

    def test_raises_value_error_for_invalid_types(self):
        with self.assertRaises(ValueError):
            to_percentage("foo")



CONTEXT_MOUNT_CODE = "/plx-context/code"
CONTEXT_MOUNT_BUILD = "/plx-context/build"
CONTEXT_MOUNT_AUTH = "/plx-context/context"
CONTEXT_MOUNT_OUTPUTS = "/plx-context/outputs"
CONTEXT_MOUNT_ARTIFACTS = "/plx-context/artifacts/{}"
CONTEXT_MOUNT_SHM = "/dev/shm"
CONTEXT_MOUNT_DOCKER = "/var/run/docker.sock"



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function


def to_unit_memory(number):
    """Creates a string representation of memory size given `number`."""
    kb = 1024

    number /= kb

    if number < 100:
        return "{} Kb".format(round(number, 2))

    number /= kb
    if number < 300:
        return "{} Mb".format(round(number, 2))

    number /= kb

    return "{} Gb".format(round(number, 2))


def to_percentage(number, rounding=2):
    """Creates a percentage string representation from the given `number`. The
    number is multiplied by 100 before adding a '%' character.

    Raises `ValueError` if `number` cannot be converted to a number.
    """
    number = float(number) * 100
    number_as_int = int(number)
    rounded = round(number, rounding)

    return "{}%".format(number_as_int if number_as_int == rounded else rounded)



POLYAXON_HEADERS_CLI_VERSION = "X_POLYAXON_CLI_VERSION"
POLYAXON_HEADERS_CLIENT_VERSION = "X_POLYAXON_CLIENT_VERSION"
POLYAXON_HEADERS_INTERNAL = "X_POLYAXON_INTERNAL"
POLYAXON_HEADERS_RUNNER = "X_POLYAXON_RUNNER"
POLYAXON_HEADERS_OPERATOR = "X_POLYAXON_OPERATOR"


class PolyaxonServices(object):  # noqa
    DOCKERIZER = "dockerizer"
    INITIALIZER = "initializer"
    SIDECAR = "sidecar"
    HELPER = "helper"
    RUNNER = "runner"
    CONTROLLER = "controller"

    VALUES = {DOCKERIZER, INITIALIZER, SIDECAR, HELPER, RUNNER, CONTROLLER}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import re

from dateutil import parser

from hestia.tz_utils import now

# pylint:disable=anomalous-backslash-in-string

DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S %Z"
ISO_DATETIME_REGEX = re.compile(
    "([0-9]+)-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])[Tt]"
    "([01][0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9]|60)(\\.[0-9]+)?"
    "(([Zz])|([\\+|\\-]([01][0-9]|2[0-3]):[0-5][0-9]))\s?"
)
DATETIME_REGEX = re.compile(
    "\d{2}(?:\d{2})?-\d{1,2}-\d{1,2}\s\d{1,2}:\d{1,2}:\d{1,2}\s\w+\s?"
)


def timestamp_search_regex(regex, log_line, convert=False):
    log_search = regex.search(log_line)
    if not log_search:
        return log_line, None

    ts = log_search.group()
    if convert:
        ts = parser.parse(ts).strftime(DATETIME_FORMAT)

    return re.sub(regex, "", log_line), ts


def log_spec(log_line, name="", timestamp=None, log_level=None, check_timestamp=True):
    if not timestamp and check_timestamp:
        log_line, timestamp = timestamp_search_regex(
            ISO_DATETIME_REGEX, log_line, convert=True
        )
        if not timestamp:
            log_line, timestamp = timestamp_search_regex(
                DATETIME_REGEX, log_line, convert=False
            )

    return "{timestamp}{log_level}{name} -- {log_line}".format(
        log_line=log_line,
        name=" {}".format(name) if name else "",
        timestamp=timestamp.strip()
        if timestamp
        else now(tzinfo=True).strftime(DATETIME_FORMAT),
        log_level=" {}".format(log_level) if log_level else "",
    )


LogSpec = log_spec


class LogLevels(object):
    INFO = "INFO"
    WARNING = "WARNING"
    ERROR = "ERROR"



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from distutils.util import strtobool  # pylint:disable=import-error


def to_bool(value, handle_none=False, exception=TypeError):
    if isinstance(value, str):
        value = strtobool(value)

    if value in (False, 0):
        return False

    if value in (True, 1):
        return True

    if handle_none and value is None:
        return False

    raise exception("The value `{}` cannot be interpreted as boolean".format(value))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function


def strip_spaces(value, sep=None, join=True):
    """Cleans trailing whitespaces and replaces also multiple whitespaces with a single space."""
    value = value.strip()
    value = [v.strip() for v in value.split(sep)]
    join_sep = sep or " "
    return join_sep.join(value) if join else value



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

try:
    import numpy as np
except ImportError:
    np = None


def sanitize_np_types(value):
    if isinstance(value, (int, float, complex, type(None))):
        return value
    if isinstance(value, np.integer):
        return int(value)
    if isinstance(value, np.floating):
        return float(value)
    return value



class AuthenticationError(Exception):
    pass


class AuthenticationTypes(object):
    TOKEN = "Token"
    INTERNAL_TOKEN = "Internaltoken"
    EPHEMERAL_TOKEN = "EphemeralToken"

    VALUES = {TOKEN, INTERNAL_TOKEN, EPHEMERAL_TOKEN}



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

try:
    import numpy as np
except ImportError:
    np = None


def to_list(value, check_none=False):
    if check_none and value is None:
        return []
    if isinstance(value, (list, tuple, set)):
        return list(value)
    if np and isinstance(value, np.ndarray):
        return value.tolist()
    return [value]



POLYAXON_KEYS_HEADER = "POLYAXON_HEADER"
POLYAXON_KEYS_HEADER_SERVICE = "POLYAXON_HEADER_SERVICE"
POLYAXON_KEYS_AUTHENTICATION_TYPE = "POLYAXON_AUTHENTICATION_TYPE"
POLYAXON_KEYS_LOG_LEVEL = "POLYAXON_LOG_LEVEL"
POLYAXON_KEYS_K8S_NAMESPACE = "POLYAXON_K8S_NAMESPACE"
POLYAXON_KEYS_API_HOST = "POLYAXON_API_HOST"
POLYAXON_KEYS_API_PORT = "POLYAXON_API_PORT"
POLYAXON_KEYS_API_VERSION = "POLYAXON_API_VERSION"
POLYAXON_KEYS_USE_HTTPS = "POLYAXON_USE_HTTPS"
POLYAXON_KEYS_VERIFY_SSL = "POLYAXON_VERIFY_SSL"
POLYAXON_KEYS_IN_CLUSTER = "POLYAXON_IN_CLUSTER"
POLYAXON_KEYS_IS_MANAGED = "POLYAXON_IS_MANAGED"
POLYAXON_KEYS_IS_LOCAL = "POLYAXON_IS_LOCAL"
POLYAXON_KEYS_IS_OFFLINE = "POLYAXON_IS_OFFLINE"
POLYAXON_KEYS_SECRET_KEY = "POLYAXON_SECRET_KEY"
POLYAXON_KEYS_SECRET_INTERNAL_TOKEN = "POLYAXON_SECRET_INTERNAL_TOKEN"
POLYAXON_KEYS_SECRET_EPHEMERAL_TOKEN = "POLYAXON_SECRET_EPHEMERAL_TOKEN"
POLYAXON_KEYS_SECRET_USER_TOKEN = "POLYAXON_SECRET_USER_TOKEN"
POLYAXON_KEYS_JOB_INFO = "POLYAXON_JOB_INFO"
POLYAXON_KEYS_ARTIFACTS_PATHS = "POLYAXON_ARTIFACTS_PATHS"
POLYAXON_KEYS_OUTPUTS_PATH = "POLYAXON_OUTPUTS_PATH"
POLYAXON_KEYS_LOGS_PATH = "POLYAXON_LOGS_PATH"
POLYAXON_KEYS_PARAMS = "POLYAXON_PARAMS"
POLYAXON_KEYS_CONTAINER_ID = "POLYAXON_CONTAINER_ID"
POLYAXON_KEYS_HEALTH_CHECK_URL = "POLYAXON_HEALTH_CHECK_URL"
POLYAXON_KEYS_RECONCILE_URL = "POLYAXON_RECONCILE_URL"
POLYAXON_KEYS_NO_OP = "POLYAXON_NO_OP"
POLYAXON_KEYS_HASH_LENGTH = "POLYAXON_HASH_LENGTH"



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import datetime

from unittest import TestCase

from hestia.tz_utils import local_now
from hestia.humanize import humanize_timedelta, humanize_timesince


class HumanizeTimesinceTest(TestCase):
    """A test case for humanize timesince"""

    def test_humanize_timesince(self):
        self.assertEqual(humanize_timesince(local_now()), "a few seconds ago")
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(minutes=1)),
            "1 minute ago",
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(minutes=10)),
            "10 minutes ago",
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(hours=1)), "1 hour ago"
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(hours=10)),
            "10 hours ago",
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(hours=24)), "1 day ago"
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(hours=72)), "3 days ago"
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(hours=168)),
            "1 week ago",
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(weeks=1)), "1 week ago"
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(weeks=3)), "3 weeks ago"
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(weeks=53)), "1 year ago"
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(days=365)), "1 year ago"
        )
        self.assertEqual(
            humanize_timesince(local_now() - datetime.timedelta(days=800)),
            "2 years ago",
        )

    def test_humanize_times_in_the_future(self):
        self.assertEqual(
            humanize_timesince(local_now() + datetime.timedelta(minutes=1)),
            "a few seconds ago",
        )

    def test_humanize_timesince_few_seconds(self):
        self.assertEqual(
            u"Last update: " + humanize_timesince(local_now()),
            u"Last update: a few seconds ago",
        )


class HumanizeTimeDeltaTest(TestCase):
    """A test case for the `humanize_timedelta`."""

    def test_works_as_expected_for_valid_values(self):
        test_data = [
            (7200, "2h"),
            (36, "36s"),
            (3600, "1h"),
            (3800, "1h 3m"),
            (33000, "9h 10m"),
            (720000, "8d 8h"),
            (1000000, "11d 13h 46m"),
        ]
        for value, expected in test_data:
            result = humanize_timedelta(value)
            self.assertEqual(result, expected)



class ManagerInterface(object):
    def __init__(self):
        self._state = {}

    @property
    def state(self):
        return self._state

    def _get_state_data(self, obj):
        raise NotImplementedError

    def subscribe(self, obj):
        key, value = self._get_state_data(obj)
        self._subscribe(key=key, value=value)

    def _subscribe(self, key, value):
        if key in self.state:
            assert self.state[key] == value
        else:
            self.state[key] = value

    def knows(self, key):
        return key in self.state

    def get(self, key):
        return self.state.get(key)

    @property
    def keys(self):
        return self.state.keys()

    @property
    def values(self):
        return self.state.values()

    @property
    def items(self):
        return self.state.items()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import pytz

from datetime import datetime

try:
    from django.utils.timezone import now as dj_now  # pylint:disable=import-error
except ImportError:
    dj_now = None


utc = pytz.utc


def get_time_zone(tz="UTC"):
    return pytz.timezone(tz)


def now(tzinfo=True):
    """
    Return an aware or naive datetime.datetime, depending on settings.USE_TZ.
    """
    if dj_now:
        return dj_now()

    if tzinfo:
        # timeit shows that datetime.now(tz=utc) is 24% slower
        return datetime.utcnow().replace(tzinfo=utc)
    return datetime.now()


def local_now(tz="UTC"):
    _tz = get_time_zone(tz=tz)
    return _tz.localize(datetime.utcnow()).replace(microsecond=0)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import pytz

from datetime import datetime, timedelta

epoch = datetime(1970, 1, 1, tzinfo=pytz.utc)


def to_timestamp(value):
    """
    Convert a time zone aware datetime to a POSIX timestamp (with fractional component.)
    """
    return (value - epoch).total_seconds()


def to_datetime(value):
    """
    Convert a POSIX timestamp to a time zone aware datetime.

    The timestamp value must be a numeric type (either a integer or float,
    since it may contain a fractional component.)
    """
    return epoch + timedelta(seconds=value)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

from hestia.exceptions import HestiaException

try:
    import requests
except ImportError:
    raise HestiaException("This module depends on requests.")


def safe_request(
    url,
    method=None,
    params=None,
    data=None,
    json=None,
    headers=None,
    allow_redirects=False,
    timeout=30,
    verify_ssl=True,
):
    """A slightly safer version of `request`."""

    session = requests.Session()

    kwargs = {}

    if json:
        kwargs["json"] = json
        if not headers:
            headers = {}
        headers.setdefault("Content-Type", "application/json")

    if data:
        kwargs["data"] = data

    if params:
        kwargs["params"] = params

    if headers:
        kwargs["headers"] = headers

    if method is None:
        method = "POST" if (data or json) else "GET"

    response = session.request(
        method=method,
        url=url,
        allow_redirects=allow_redirects,
        timeout=timeout,
        verify=verify_ssl,
        **kwargs
    )

    return response



from unittest import TestCase

from hestia.list_utils import to_list


class ToListTest(TestCase):
    def test_to_list(self):
        assert to_list(None) == [None]
        assert to_list(None, check_none=True) == []
        assert to_list([]) == []
        assert to_list(()) == []
        assert to_list([1, 3]) == [1, 3]
        assert to_list((1, 3)) == [1, 3]
        assert to_list(1) == [1]
        assert to_list("foo") == ["foo"]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os


def polyaxon_user_path():
    base_path = os.path.expanduser("~")
    if not os.access(base_path, os.W_OK):
        base_path = "/tmp"

    return os.path.join(base_path, ".polyaxon")



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import os
import shutil


def move_recursively(src, dst):
    files = os.listdir(src)

    for f in files:
        shutil.move(os.path.join(src, f), dst)



class StoreTypes(object):
    HOST_PATH = 'host_path'
    VOLUME_CLAIM = 'volume_claim'
    GCS = 'gcs'
    S3 = 's3'
    AZURE = 'azure'

    VALUES = {HOST_PATH, VOLUME_CLAIM, GCS, S3, AZURE}
    CLOUD_STORES = {GCS, S3, AZURE}

    CHOICES = (
        (HOST_PATH, HOST_PATH),
        (VOLUME_CLAIM, VOLUME_CLAIM),
        (GCS, GCS),
        (S3, S3),
        (AZURE, AZURE)
    )

    @classmethod
    def is_bucket(cls, store_type):
        return store_type in cls.CLOUD_STORES



#!/usr/bin/env python

import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


def read_readme():
    with open("README.md") as f:
        return f.read()


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        errcode = pytest.main(self.test_args)
        sys.exit(errcode)


setup(
    name="hestia",
    version="0.4.4",
    description="Polyaxon common utilities.",
    long_description=read_readme(),
    long_description_content_type="text/markdown",
    maintainer="Mourad Mourafiq",
    maintainer_email="mourad@polyaxon.com",
    author="Mourad Mourafiq",
    author_email="mourad@polyaxon.com",
    url="https://github.com/polyaxon/hestia",
    license="MIT",
    platforms="any",
    packages=find_packages(),
    keywords=["polyaxon", "django"],
    install_requires=[],
    classifiers=[
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP",
    ],
    tests_require=["pytest"],
    cmdclass={"test": PyTest},
)



from hestia.exceptions import HestiaException

try:
    from urllib.parse import urlparse  # pylint:disable=import-error
except ImportError:
    raise HestiaException("This module depends on python3.")


def validate_url(url):
    if not url.startswith(("http://", "https://")):
        return False
    parsed = urlparse(url)
    if not parsed.hostname:
        return False
    return True



from hestia.exceptions import HestiaException

try:
    from rest_framework import HTTP_HEADER_ENCODING
except ImportError:
    raise HestiaException("This module depends on django-rest-framework.")


def get_header(request, header_service):
    """Return request's 'X_POLYAXON_...:' header, as a bytestring.

    Hide some test client ickyness where the header can be unicode.
    """
    service = request.META.get("HTTP_{}".format(header_service), b"")
    if isinstance(service, str):
        # Work around django test client oddness
        service = service.encode(HTTP_HEADER_ENCODING)
    return service
